package com.bewo.admedia.executors;

import android.content.Context;
import android.os.AsyncTask;
import android.telephony.TelephonyManager;
import android.util.Log;

import com.bewo.admedia.utils.AppConstants;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.apache.http.util.EntityUtils;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * Created by bewopos team.
 */
public class DownloadVideoExecutors extends AsyncTask<String, Integer, Boolean> {

    private Context context;

    public DownloadVideoExecutors(Context ctxt) {
        context = ctxt;
    }

    @Override
    protected Boolean doInBackground(String... params) {
        String result = "";

        HttpParams params_t = new BasicHttpParams();
        HttpConnectionParams.setConnectionTimeout(params_t, 15000);
        HttpConnectionParams.setSoTimeout(params_t, 15000);

        HttpClient httpclient = new DefaultHttpClient(params_t);

        StringBuilder builder = new StringBuilder();

        TelephonyManager tm = (TelephonyManager)context.getSystemService(Context.TELEPHONY_SERVICE);
        System.out.println("IMEI : " + tm.getDeviceId());

        HttpGet httpGet = new HttpGet(AppConstants.URL+":"+AppConstants.Port+AppConstants.VideosSubURL+tm.getDeviceId()+".json");

        try {

            HttpResponse httpResponse = httpclient.execute(httpGet);

            final int responseStatusCode = httpResponse.getStatusLine().getStatusCode();

            if(responseStatusCode == 200) {

                HttpEntity httpResponseEntity = httpResponse.getEntity();

                result = EntityUtils.toString(httpResponseEntity);

                /*httpInputStream = httpResponseEntity.getContent();
                BufferedReader reader = new BufferedReader(new InputStreamReader(httpInputStream));
                String line;
                while ((line = reader.readLine()) != null) {
                    builder.append(line);
                }
                httpInputStream.close();
                result = builder.toString();*/

                System.out.println("Response : " + result);

                return readVideosList(result);

            } else {
                System.out.println("Response Status : " + responseStatusCode);
            }

        } catch (Exception exception) {
            Log.d("Error", exception.getLocalizedMessage());
            return false;
        }
        return false;
    }

    @Override
    protected void onPostExecute(Boolean result) {
        if(result) {
            System.out.println("Request Completed...");
        } else {
            System.out.println("Request return false..!!!");
        }
    }

    private boolean readVideosList(String response) {

        final JSONArray geodata;
        try {
            geodata = new JSONArray(response);
            final int n = geodata.length();
            for (int i = 0; i < n; ++i) {
                final JSONObject person = geodata.getJSONObject(i);
                int videoId = person.getInt("id");
                String strVideoFilename = person.getString("video_file_name");
                File file = doGetFile(videoId, strVideoFilename);
                if(!file.exists()) {
                    System.out.println("File Name : " + strVideoFilename+ " not WRITTEN.");
                }
            }
            return true;
        } catch (JSONException e) {
            e.printStackTrace();
            return false;
        }
    }

    public File doGetFile(int videoId, String strVideoFilename) {

        File file = null;

        HttpClient httpclient = new DefaultHttpClient();

        HttpGet httpget = new HttpGet(AppConstants.URL+":"+AppConstants.Port+AppConstants.VideoSubURL+videoId);

        HttpResponse response;

        try {
            response = httpclient.execute(httpget);

            final int responseStatusCode = response.getStatusLine().getStatusCode();

            if(responseStatusCode == 200) {

                HttpEntity entity = response.getEntity();

                if (entity != null) {

                    InputStream instream = entity.getContent();

                    file = convertStreamToFile(strVideoFilename, instream);

                    instream.close();
                } else {
                    System.out.println("Entity null");
                    return null;
                }
            } else {
                System.out.println("Video Request : "+responseStatusCode);
                return null;
            }
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        // Return the jsonArray
        return file;
    }

    private static File convertStreamToFile(String strVideoFileName, InputStream is) {

        byte[] outputByte = new byte[1024];

        double chunkBytes = 0;
        double bytesRead = 0;

        FileOutputStream fileOutputStream = null;

        File fileDownloaded = new File(AppConstants.AdVideosPath + strVideoFileName);

        if(fileDownloaded != null) {
            try {
                fileOutputStream = new FileOutputStream(fileDownloaded);
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            }

            try {
                while ((chunkBytes = is.read(outputByte, 0, 1024)) != -1) {
                    try{
                        bytesRead = bytesRead + chunkBytes;
                        fileOutputStream.write(outputByte, 0, (int) chunkBytes);
                    } catch(Exception ex){
                        Log.i("Downloading", "Video file download error.");
                    }
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
            try {
                is.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            try {
                fileOutputStream.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return fileDownloaded;
    }

}
