package com.bewo.admedia.utils;

import android.content.Context;
import android.graphics.Paint;
import android.text.TextUtils;
import android.util.AttributeSet;
import android.util.Log;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.LinearInterpolator;
import android.view.animation.TranslateAnimation;
import android.widget.LinearLayout;
import android.widget.ScrollView;
import android.widget.TextView;

import com.bewo.admedia.R;
import com.bewo.admedia.listener.AdTextListener;


/**
 * Created by bewopos team.
 */
public class AutomaticScrollView extends LinearLayout {

    private static final String TAG = "AutomaticScrollView";
    // Context of application
    Context context;
    // TextView
    private TextView mTextField1;

    // Horizontal scroll
    private ScrollView mScrollView1;

    // Animation on start
    private Animation mMoveTextOnStart = null;
    // Out animation
    private Animation mMoveText1TextOut = null;

    // Duration of animation on start
    private int durationStart;
    // Duration of animation
    private int duration;

    // Pain for drawing text
    private Paint mPaint;

    // Text current width
    private float mText1TextWidth;

    /**
     * Control the speed. The lower this value, the faster it will scroll.
     */
    public static final int MS_PER_PX = 8;

    /**
     * Control the pause between the animations. Also, after starting this
     * activity.
     */
    public static final int PAUSE_BETWEEN_ANIMATIONS = 0;
    private boolean mCancelled = false;

    // Layout width
    private int mWidth;
    // Animation thread
    private Runnable mAnimation1StartRunnable;

    private AdTextListener adTextListener;

    public AutomaticScrollView(Context context, AdTextListener listener) {
        super(context);
        init(context);
        this.context = context;
        adTextListener = listener;
    }

    public AutomaticScrollView(Context context, AttributeSet attrs) {
        super(context, attrs);
        init(context);
        this.context = context;
        adTextListener = (AdTextListener) context;
    }

    private void init(Context context) {
        initView(context);

        // init helper
        mPaint = new Paint();
        mPaint.setAntiAlias(true);
        mPaint.setStrokeWidth(1);
        mPaint.setStrokeCap(Paint.Cap.ROUND);

    }

    @Override
    protected void onLayout(boolean changed, int l, int t, int r, int b) {
        super.onLayout(changed, l, t, r, b);

        mWidth = getMeasuredWidth();

        // Calculate
        prepare();

        // Setup
        //setupText1Marquee();

    }

    @Override
    public void setOnClickListener(OnClickListener l) {
        super.setOnClickListener(l);

        mTextField1.setOnClickListener(l);
    }

    // Method to finally start the marquee.
    public void startMarquee() {
        prepare();
        prepareTextFields();

        mTextField1.setVisibility(View.VISIBLE);
        //mTextField1.startAnimation(mMoveTextOnStart);

        //startTextField1Animation();

    }

    public void prepareTextFields() {
        expandTextView(mTextField1);
    }

    private void prepare() {

        // Measure
        mPaint.setTextSize(mTextField1.getTextSize());
        mPaint.setTypeface(mTextField1.getTypeface());
        mText1TextWidth = mPaint.measureText(mTextField1.getText().toString());

        setupText1Marquee();

    }

    private void setupText1Marquee() {

        // Calculate duration of animations
        durationStart = (int) ((mWidth + mText1TextWidth) * MS_PER_PX);
        duration = (int) (2 * mWidth * MS_PER_PX);

        System.out.println("Duration Start : ("+mWidth+" + "+mText1TextWidth+") * "+MS_PER_PX + " = "+durationStart);
        //System.out.println("Duration : 2 * "+mWidth+" * "+MS_PER_PX+" = "+duration);

        // On start animation
        mMoveTextOnStart = new TranslateAnimation(0,-mWidth - mText1TextWidth, 0, 0);

        mMoveTextOnStart.setDuration(durationStart);
        mMoveTextOnStart.setInterpolator(new LinearInterpolator());
        mMoveTextOnStart.setFillAfter(true);

        // Main scrolling animation
        mMoveText1TextOut = new TranslateAnimation(mWidth, - mText1TextWidth, 0, 0);

        mMoveText1TextOut.setDuration(duration);
        mMoveText1TextOut.setInterpolator(new LinearInterpolator());
        mMoveText1TextOut.setFillAfter(true);
        mMoveText1TextOut.setRepeatCount(Animation.INFINITE);

        // Animation listeners
        mMoveTextOnStart
                .setAnimationListener(new Animation.AnimationListener() {
                    public void onAnimationStart(Animation animation) {
                        Log.d(TAG, "TextIN - START");
                        //invalidate();
                        //mTextField1.invalidate();

                    }

                    public void onAnimationEnd(Animation animation) {
                        Log.d(TAG, "TextIN - END");

                        mTextField1.startAnimation(mMoveText1TextOut);
                    }

                    public void onAnimationRepeat(Animation animation) {
                        Log.d(TAG, "TextIN - Repeat");
                        //invalidate();
                        //mTextField1.invalidate();
                        //adTextListener.onTextAnimationFinished();
                    }
                });

        mMoveText1TextOut
                .setAnimationListener(new Animation.AnimationListener() {
                    public void onAnimationStart(Animation animation) {
                        Log.d(TAG, "TextOUT - START");
                        //invalidate();
                        //mTextField1.invalidate();

                    }

                    public void onAnimationEnd(Animation animation) {
                        Log.d(TAG, "TextOut - END");
                        //invalidate();
                    }

                    public void onAnimationRepeat(Animation animation) {
                        Log.d(TAG, "TextOut - Repeat");
                        //invalidate();
                        //mTextField1.invalidate();
                        adTextListener.onTextAnimationFinished();
                    }
                });

    }


    private void initView(Context context) {
        setOrientation(LinearLayout.HORIZONTAL);
        setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT,
                LayoutParams.FILL_PARENT, Gravity.LEFT));
        setPadding(0, 0, 0, 0);
        setGravity(Gravity.CENTER_VERTICAL);

        // Scroll View 1
        LayoutParams sv1lp = new LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT);
        //sv1lp.gravity = Gravity.CENTER_HORIZONTAL;
        mScrollView1 = new ScrollView(context);

        // Scroll View 1 - Text Field
        mTextField1 = new TextView(context);
        mTextField1.setSingleLine(true);
        mTextField1.setTextColor(context.getResources().getColor(R.color.white));
        // mTextField1.setEllipsize(TextUtils.TruncateAt.END);
        // mTextField1.setTypeface(null, Typeface.BOLD);

        System.out.println("Scroll Width " + mTextField1.getWidth());
        mScrollView1.addView(mTextField1, new ScrollView.LayoutParams(mTextField1.getWidth(), LayoutParams.WRAP_CONTENT));

        addView(mScrollView1, sv1lp);
    }

    public void setText(String text) {
        mTextField1.setText(text);
        System.out.println("Text : "+text);
        prepareTextFields();
        prepare();
    }

    private void expandTextView(TextView textView) {
        ViewGroup.LayoutParams lp = textView.getLayoutParams();
        mPaint.setTextSize(mTextField1.getTextSize());
        mPaint.setTypeface(mTextField1.getTypeface());
        System.out.println("Text Length : " + (int) mPaint.measureText(mTextField1.getText().toString()));
        lp.width = (int) mPaint.measureText(mTextField1.getText().toString());;
        //(int) mPaint.measureText(mTextField1.getText().toString());
        // LayoutParams.MATCH_PARENT;
        // AutomaticScrollActivity.getScreenWidth();
        textView.setLayoutParams(lp);
    }


    public void setTextSize(int textSize) {
        mTextField1.setTextSize(textSize);

    }

    public void setTextColor1(int textColor) {

        mTextField1.setTextColor(textColor);
    }

    public void reset() {

        mCancelled = true;

        if (mAnimation1StartRunnable != null) {
            removeCallbacks(mAnimation1StartRunnable);
        }

        mTextField1.clearAnimation();

        prepareTextFields();

        mMoveTextOnStart.reset();
        mMoveText1TextOut.reset();

        mScrollView1.removeView(mTextField1);
        mScrollView1.addView(mTextField1);

        mTextField1.setEllipsize(TextUtils.TruncateAt.END);

        invalidate();
    }
}